/*
How to generate an OAuth token:
- Go to https://twitchapps.com/tmi/ and connect it to the account you want to use with this base (most likely a second bot account).

How to use this for your own plugins using chat:
- add "TwitchBase" to the dependencies array in the info.toml of your plugin!

Also, refer to the Twitch IRC documentation for more info on tags: https://dev.twitch.tv/docs/irc/tags
*/

namespace Twitch { // in your Plugin, you will need to call these methods with the prefix "Twitch::"

    // In your plugin, call this method once at the start together with the type of messages you want to get updates about!
    // @returns your plugin Id for this plugin to recognize you. Save this, you will use this to get new messages.
    import int Register(const uint updateType) from "TwitchBase";

    // Call this method together with the integer you received at registering to unregister from updates.
    // You could try that for example in the OnDisabled plugin function. Remember to re-register tho!
    // @returns true if successful
    import bool Unregister(const int pluginId) from "TwitchBase";

    // Call this method together with the integer you received at registering. Make sure to request more than 1 time a second!
    // @returns an array of Twitch::Messages, filtered by the flags you set earlier. Definition in Structures.as
    import array<Message@> Fetch(const int pluginId) from "TwitchBase";

    // Call this method to send a message into the connected twitch channel.
    // This is currently limited to the standard limits for all channels (20 msgs / 30 seconds)
    // If you send messages in a channel the user is moderator / broadcaster in, you can go up to 100 msgs / 30 seconds
    // However, this is not implemented. If your plugin really needs that bandwith, let me know, there will be a plugin update managing this transparently
    // @returns MessageCode (informs you of success, if its rate limited or if the plugin connection doesnt even exist)
    import MessageCode SendMessage(const string &in channel, const string &in msg) from "TwitchBase";

    // Call this method to whisper a message to a twitch user.
    // P. S.: The user will need to explicitly enable whispers using external IRC for this to be actually sent. Be careful!
    // P. P. S.: One must be connected to at least one channel to whisper. Make sure to check!
    // @returns MessageCode (informs you of success, if its rate limited or if the plugin connection doesnt even exist)
    import MessageCode SendWhisper(const string &in user, const string &in msg) from "TwitchBase";

    // Call this method to get a list of all joined channels, from which it is safe to check ROOMSTATE / USERSTATE of (they are either set from twitch or not null)
    // WARNING: Will not be complete if called before we checked all channels, CHECK ChannelsJoined() !
    // @returns an array of channel states for all plugins listed in the plugin settings to join, whether successful or not
    import array<ChannelState@> GetJoinedChannels() from "TwitchBase";

    // @returns true if GetJoinedChannels will return a full result (= all requested channels have been checked as existing or not) and it is safe to request the USERSTATE / ROOMSTATE.
    import bool ChannelsJoined() from "TwitchBase";

    // Call this method to get a list of all requested channels, whether they exist or not and joined or not
    // WARNING: Will be empty if called before we connected to twitch, CHECK ChannelsAdded() !
    // @returns an array of channel states for all plugins listed in the plugin settings to join, whether successful or not
    import array<ChannelState@> GetAllChannels() from "TwitchBase";

    // @returns true if GetAllChannels will return a full result (= all requested channels have been added to that array)
    import bool ChannelsAdded() from "TwitchBase";

    // Call this method to get the username of the connected account.
    // (Can also be achieved in other ways, like userstate, so it's just here for simplicity)
    // @returns the username set in the plugin settings
    import string GetUsername() from "TwitchBase";
}